@@ -14,6 +14,7 @@ A library of fundamental behaviours for use.
 # Imports
 ##############################################################################
 
+import copy
 import functools
 import operator
 import typing
@@ -116,6 +117,37 @@ class Periodic(behaviour.Behaviour):
         return self.response
 
 
+class StatusSequence(behaviour.Behaviour):
+    """
+    Cycle through the specified sequence of states.
+
+    Args:
+        name: name of the behaviour
+        sequence: list of status values to cycle through
+        eventually: status to use eventually, None to re-cycle the sequence
+    """
+    def __init__(
+            self,
+            name: str,
+            sequence: typing.List[common.Status],
+            eventually: typing.Optional[common.Status]
+    ):
+        super(StatusSequence, self).__init__(name)
+        self.sequence = sequence
+        self.eventually = eventually
+        self.current_sequence = copy.copy(sequence)
+
+    def update(self):
+        if self.current_sequence:
+            status = self.current_sequence.pop(0)
+        elif self.eventually is not None:
+            status = self.eventually
+        else:
+            self.current_sequence = copy.copy(self.sequence)
+            status = self.current_sequence.pop(0)
+        return status
+
+
 class SuccessEveryN(behaviour.Behaviour):
     """
     This behaviour updates it's status with :data:`~py_trees.common.Status.SUCCESS`
